==> v1.0.0.tar.gz: platomix-gmetry-bi/platomix-gmetry-bi-api/src/main/java/org/springblade/visual/service/VisualComponentExportService.java <==
/**
 * 
 */
package org.springblade.visual.service;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.entity.VisualComponentExport;
import org.springblade.visual.mapper.VisualComponentExportMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.converters.longconverter.LongStringConverter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.Tuple;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Service
@SuppressWarnings("unchecked")
public class VisualComponentExportService extends ServiceImpl<VisualComponentExportMapper, VisualComponentExport> {

	@Value("${blade.file.exportComponent}")
	private String exportComponentDir;

	@Autowired
	private DynamicTableService dynamicTableService;

	@Async
	public void doExport(VisualComponentExport visualComponentExport, QuerySqlParam querySqlParam) {
		ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
		String querySql = querySqlParam.getQuerySql();
		List<ComponentColumn> columnList = querySqlParam.getColumnList();
		File excelFile = buildFile(visualComponentExport.getFileName(), visualComponentExport.getVisualId());
		Tuple tuple = _buildHeader(columnList, querySqlParam.getExcludeColumnIdList());
		ExcelWriter excelWriter = EasyExcel.write(excelFile)
				.registerConverter(new LongStringConverter())
				.head((List<List<String>>) tuple.get(0)).build();
		List<String> headerColumnAliasList = (List<String>) tuple.get(1);
		WriteSheet writeSheet = EasyExcel.writerSheet(0, "sheet1").build();
		String errMsg = null;
		long startTime = System.currentTimeMillis();
		// 页码和每页条数
		try {
			long current = 0;
			int limitVal = 3000;
			while (true) {
				IPage<Map<String, Object>> pageData = dynamicTableService.selectPage(new Page<>(current, limitVal), reportDatasource,
						querySql);
				if (pageData != null && CollectionUtil.isNotEmpty(pageData.getRecords())) {
					List<Map<String, Object>> resultList = pageData.getRecords();
					//封装数据
					List<List<Object>> dataList = _buildDataList(headerColumnAliasList, resultList);
					// 往excel写数据
					excelWriter.write(dataList, writeSheet);
					current = pageData.getCurrent() + 1;
				} else {
					break; // 无数据，跳出循环
				}
			}
		} catch (Exception e) {
			errMsg = e.getMessage();
			log.error(e.getMessage(), e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
			VisualComponentExport entity = new VisualComponentExport();
			entity.setId(visualComponentExport.getId());
			entity.setExportStatus(errMsg == null ? 1 : 2);
			entity.setExt(errMsg);
			entity.setFinishTime(new Date());
			updateById(entity);
			log.info("数据已导出｜耗时：{}s", (System.currentTimeMillis() - startTime) / 1000.f);
		}
	}

	private List<List<Object>> _buildDataList(List<String> headerColumnAliasList, List<Map<String, Object>> resultList) {
		List<List<Object>> dataList = new ArrayList<List<Object>>();
		for (Map<String, Object> jsonObj : resultList) {
			List<Object> rowList = new ArrayList<>();
			headerColumnAliasList.forEach(item -> {
				if (jsonObj.containsKey(item)) {
					Object value = jsonObj.get(item);
					rowList.add(_formatObjectValue(value));
				}
			});
			dataList.add(rowList);
		}
		return dataList;
	}

	public File buildFile(String fileName, Integer visualId) {
		File dir = FileUtil.file(exportComponentDir, "" + visualId);
		dir.mkdirs();
		return FileUtil.file(dir, fileName);
	}

	private Tuple _buildHeader(List<ComponentColumn> columnList, Set<Integer> excludeColumnIdList) {
		List<List<String>> headerList = new ArrayList<List<String>>();
		List<String> headerColumnAliasList = new ArrayList<>();
		columnList.forEach(value -> {
			if (CollectionUtil.isEmpty(excludeColumnIdList) || excludeColumnIdList.contains(value.getCubeColumnId())) {
				headerList.add(new ArrayList<>(Arrays.asList(value.getColumnTitle())));
				headerColumnAliasList.add(value.getColumnAliasName());
			}
		});
		return new Tuple(headerList, headerColumnAliasList);
	}
	
	/**
	 * 格式化数据
	 * @param objValue
	 * @return
	 */
	private Object _formatObjectValue(Object objValue) {
		if(objValue == null) {
			return "";
		}
		if(objValue instanceof java.sql.Timestamp) {
			return DateUtil.format((java.sql.Timestamp) objValue, DatePattern.NORM_DATETIME_PATTERN);
		}else if(objValue instanceof java.sql.Date){
			return DateUtil.format((java.sql.Date) objValue, DatePattern.NORM_DATETIME_PATTERN);	
		}
		return objValue;
	}
	
}

==> v3.4.1.tar.gz: platomix-gmetry-bi/platomix-gmetry-bi-api/src/main/java/org/springblade/visual/service/VisualComponentExportService.java <==
/**
 * 
 */
package org.springblade.visual.service;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.entity.VisualComponentExport;
import org.springblade.visual.mapper.VisualComponentExportMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.converters.longconverter.LongStringConverter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.Tuple;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Service
@SuppressWarnings("unchecked")
public class VisualComponentExportService extends ServiceImpl<VisualComponentExportMapper, VisualComponentExport> {

	@Value("${blade.file.exportComponent}")
	private String exportComponentDir;

	@Autowired
	private DynamicTableService dynamicTableService;

	@Async
	public void doExport(VisualComponentExport visualComponentExport, QuerySqlParam querySqlParam) {
		ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
		String querySql = querySqlParam.getQuerySql();
		List<ComponentColumn> selectColumnList = querySqlParam.getSelectColumnList();
		File excelFile = buildFile(visualComponentExport.getFileName(), visualComponentExport.getVisualId());
		Tuple tuple = _buildHeader(selectColumnList, querySqlParam.getExcludeColumnIdList());
		ExcelWriter excelWriter = EasyExcel.write(excelFile)
				.registerConverter(new LongStringConverter())
				.head((List<List<String>>) tuple.get(0)).build();
		List<String> headerColumnAliasList = (List<String>) tuple.get(1);
		WriteSheet writeSheet = EasyExcel.writerSheet(0, "sheet1").build();
		String errMsg = null;
		long startTime = System.currentTimeMillis();
		// 页码和每页条数
		try {
			long current = 0;
			int limitVal = 3000;
			while (true) {
				IPage<Map<String, Object>> pageData = dynamicTableService.selectPage(new Page<>(current, limitVal), reportDatasource,
						querySql);
				if (pageData != null && CollectionUtil.isNotEmpty(pageData.getRecords())) {
					List<Map<String, Object>> resultList = pageData.getRecords();
					//封装数据
					List<List<Object>> dataList = _buildDataList(headerColumnAliasList, resultList);
					// 往excel写数据
					excelWriter.write(dataList, writeSheet);
					current = pageData.getCurrent() + 1;
				} else {
					break; // 无数据，跳出循环
				}
			}
		} catch (Exception e) {
			errMsg = e.getMessage();
			log.error(e.getMessage(), e);
		} finally {
			if (excelWriter != null) {
				excelWriter.finish();
			}
			VisualComponentExport entity = new VisualComponentExport();
			entity.setId(visualComponentExport.getId());
			entity.setExportStatus(errMsg == null ? 1 : 2);
			entity.setExt(errMsg);
			entity.setFinishTime(new Date());
			updateById(entity);
			log.info("数据已导出｜耗时：{}s", (System.currentTimeMillis() - startTime) / 1000.f);
		}
	}

	private List<List<Object>> _buildDataList(List<String> headerColumnAliasList, List<Map<String, Object>> resultList) {
		List<List<Object>> dataList = new ArrayList<List<Object>>();
		for (Map<String, Object> jsonObj : resultList) {
			List<Object> rowList = new ArrayList<>();
			headerColumnAliasList.forEach(item -> {
				if (jsonObj.containsKey(item)) {
					Object value = jsonObj.get(item);
					rowList.add(_formatObjectValue(value));
				}
			});
			dataList.add(rowList);
		}
		return dataList;
	}

	public File buildFile(String fileName, Integer visualId) {
		File dir = FileUtil.file(exportComponentDir, "" + visualId);
		dir.mkdirs();
		return FileUtil.file(dir, fileName);
	}

	private Tuple _buildHeader(List<ComponentColumn> selectColumnList, Set<Integer> excludeColumnIdList) {
		List<List<String>> headerList = new ArrayList<List<String>>();
		List<String> headerColumnAliasList = new ArrayList<>();
		selectColumnList.forEach(value -> {
			if (CollectionUtil.isEmpty(excludeColumnIdList) || excludeColumnIdList.contains(value.getCubeColumnId())) {
				headerList.add(new ArrayList<>(Arrays.asList(value.getColumnTitle())));
				headerColumnAliasList.add(value.getColumnAliasName());
			}
		});
		return new Tuple(headerList, headerColumnAliasList);
	}
	
	/**
	 * 格式化数据
	 * @param objValue
	 * @return
	 */
	private Object _formatObjectValue(Object objValue) {
		if(objValue == null) {
			return "";
		}
		if(objValue instanceof java.sql.Timestamp) {
			return DateUtil.format((java.sql.Timestamp) objValue, DatePattern.NORM_DATETIME_PATTERN);
		}else if(objValue instanceof java.sql.Date){
			return DateUtil.format((java.sql.Date) objValue, DatePattern.NORM_DATETIME_PATTERN);	
		}
		return objValue;
	}
	
}
